@@ -39,10 +39,12 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	registryv1 "github.com/openshift/api/imageregistry/v1"
 	networkv1 "github.com/openshift/api/network/v1"
+	authv1client "github.com/openshift/client-go/authorization/clientset/versioned/typed/authorization/v1"
 	openshiftscheme "github.com/openshift/client-go/config/clientset/versioned/scheme"
 	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	imageregistryv1 "github.com/openshift/client-go/imageregistry/clientset/versioned/typed/imageregistry/v1"
 	networkv1client "github.com/openshift/client-go/network/clientset/versioned/typed/network/v1"
+	securityv1client "github.com/openshift/client-go/security/clientset/versioned/typed/security/v1"
 	"github.com/openshift/library-go/pkg/image/reference"
 	restclient "k8s.io/client-go/rest"
 
@@ -82,8 +84,9 @@ const (
 	InstallPlansTopX = 100
 
 	// Maximal total number of service accounts
-	maxServiceAccountsLimit = 1000
-	maxNamespacesLimit      = 1000
+	maxServiceAccountsLimit          = 1000
+	maxNamespacesLimit               = 1000
+	maxServiceAccountNamespacesLimit = 1000
 )
 
 var (
@@ -106,6 +109,8 @@ var (
 
 	// lineSep is the line separator used by the alerts metric
 	lineSep = []byte{'\n'}
+
+	defaultNamespaces = []string{"default", "kube-system", "kube-public"}
 )
 
 func init() {
@@ -130,6 +135,8 @@ type Gatherer struct {
 	lock            sync.Mutex
 	lastVersion     *configv1.ClusterVersion
 	discoveryClient discovery.DiscoveryInterface
+	authClient      authv1client.AuthorizationV1Interface
+	securityClient  securityv1client.SecurityV1Interface
 }
 
 type clusterOperatorResource struct {
@@ -139,9 +146,15 @@ type clusterOperatorResource struct {
 	Spec       interface{}
 }
 
+type netNamespace struct {
+	Name      string                           `json:"name"`
+	EgressIPs []networkv1.NetNamespaceEgressIP `json:"egressIPs"`
+	NetID     uint32                           `json:"netID"`
+}
+
 // New creates new Gatherer
 func New(client configv1client.ConfigV1Interface, coreClient corev1client.CoreV1Interface, certClient certificatesv1beta1.CertificatesV1beta1Interface, metricsClient rest.Interface,
-	registryClient imageregistryv1.ImageregistryV1Interface, crdClient apixv1beta1client.ApiextensionsV1beta1Interface, networkClient networkv1client.NetworkV1Interface, dynamicClient dynamic.Interface, policyClient policyclient.PolicyV1beta1Interface, discoveryClient discovery.DiscoveryInterface) *Gatherer {
+	registryClient imageregistryv1.ImageregistryV1Interface, crdClient apixv1beta1client.ApiextensionsV1beta1Interface, networkClient networkv1client.NetworkV1Interface, dynamicClient dynamic.Interface, policyClient policyclient.PolicyV1beta1Interface, discoveryClient discovery.DiscoveryInterface, authClient authv1client.AuthorizationV1Interface, securityClient securityv1client.SecurityV1Interface) *Gatherer {
 	return &Gatherer{
 		client:          client,
 		coreClient:      coreClient,
@@ -153,6 +166,8 @@ func New(client configv1client.ConfigV1Interface, coreClient corev1client.CoreV1
 		dynamicClient:   dynamicClient,
 		policyClient:    policyClient,
 		discoveryClient: discoveryClient,
+		authClient:      authClient,
+		securityClient:  securityClient,
 	}
 }
 
@@ -187,6 +202,9 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 		GatherInstallPlans(i),
 		GatherContainerRuntimeConfig(i),
 		GatherOpenshiftSDNLogs(i),
+		GatherNetNamespace(i),
+		GatherServiceAccounts(i),
+		GatherSAPConfig(i),
 	)
 }
 
@@ -776,6 +794,8 @@ func GatherClusterImagePruner(i *Gatherer) func() ([]record.Record, []error) {
 }
 
 // GatherClusterImageRegistry fetches the cluster Image Registry configuration
+// If the Image Registry configuration uses some PersistentVolumeClaim for the storage then the corresponding
+// PersistentVolume definition is gathered
 //
 // Location in archive: config/clusteroperator/imageregistry.operator.openshift.io/config/cluster.json
 func GatherClusterImageRegistry(i *Gatherer) func() ([]record.Record, []error) {
@@ -787,6 +807,22 @@ func GatherClusterImageRegistry(i *Gatherer) func() ([]record.Record, []error) {
 		if err != nil {
 			return nil, []error{err}
 		}
+		records := []record.Record{}
+		// if there is some PVC then try to gather used persistent volume
+		if config.Spec.Storage.PVC != nil {
+
+			pvcName := config.Spec.Storage.PVC.Claim
+			pv, err := findPVByPVCName(i.ctx, i.coreClient, pvcName)
+			if err != nil {
+				klog.Errorf("unable to find persistent volume: %s", err)
+			} else {
+				pvRecord := record.Record{
+					Name: fmt.Sprintf("config/persistentvolumes/%s", pv.Name),
+					Item: PersistentVolumeAnonymizer{pv},
+				}
+				records = append(records, pvRecord)
+			}
+		}
 		// TypeMeta is empty - see https://github.com/kubernetes/kubernetes/issues/3030
 		kinds, _, err := registryScheme.ObjectKinds(config)
 		if err != nil {
@@ -796,10 +832,13 @@ func GatherClusterImageRegistry(i *Gatherer) func() ([]record.Record, []error) {
 			klog.Warningf("More kinds for image registry config operator resource %s", kinds)
 		}
 		objKind := kinds[0]
-		return []record.Record{{
+		coRecord := record.Record{
 			Name: fmt.Sprintf("config/clusteroperator/%s/%s/%s", objKind.Group, strings.ToLower(objKind.Kind), config.Name),
 			Item: ImageRegistryAnonymizer{config},
-		}}, nil
+		}
+
+		records = append(records, coRecord)
+		return records, nil
 	}
 }
 
@@ -1006,6 +1045,7 @@ func GatherMachineConfigPool(i *Gatherer) func() ([]record.Record, []error) {
 		return records, nil
 	}
 }
+
 // GatherContainerRuntimeConfig collects ContainerRuntimeConfig  information
 //
 // The Kubernetes api https://github.com/openshift/machine-config-operator/blob/master/pkg/apis/machineconfiguration.openshift.io/v1/types.go#L402
@@ -1097,6 +1137,106 @@ func GatherInstallPlans(i *Gatherer) func() ([]record.Record, []error) {
 	}
 }
 
+// GatherNetNamespace collects NetNamespaces networking information
+//
+// The Kubernetes api https://github.com/openshift/client-go/blob/master/network/clientset/versioned/typed/network/v1/netnamespace.go
+// Response is an array of netNamespaces. Netnamespace contains Name, EgressIPs and NetID attributes.
+//
+// Location in archive: config/netnamespaces
+// Id in config: netnamespaces
+func GatherNetNamespace(g *Gatherer) func() ([]record.Record, []error) {
+	return func() ([]record.Record, []error) {
+		nsList, err := g.networkClient.NetNamespaces().List(g.ctx, metav1.ListOptions{})
+		if errors.IsNotFound(err) {
+			return nil, nil
+		}
+		if err != nil {
+			return nil, []error{err}
+		}
+		namespaces := []*netNamespace{}
+		for _, n := range nsList.Items {
+			netNS := &netNamespace{
+				Name:      n.Name,
+				EgressIPs: n.EgressIPs,
+				NetID:     n.NetID,
+			}
+			namespaces = append(namespaces, netNS)
+		}
+		r := record.Record{
+			Name: "config/netnamespaces",
+			Item: NetNamespaceAnonymizer{namespaces: namespaces},
+		}
+		return []record.Record{r}, nil
+	}
+}
+
+// GatherSAPConfig collects selected security context constraints
+// and cluster role bindings from clusters running a SAP payload.
+//
+// Relevant OpenShift API docs:
+//   - https://pkg.go.dev/github.com/openshift/client-go/authorization/clientset/versioned/typed/authorization/v1
+//   - https://pkg.go.dev/github.com/openshift/client-go/security/clientset/versioned/typed/security/v1
+//
+// Location in archive: config/securitycontentconstraint/, config/clusterrolebinding/
+func GatherSAPConfig(i *Gatherer) func() ([]record.Record, []error) {
+	return func() ([]record.Record, []error) {
+		sccToGather := []string{"anyuid", "privileged"}
+		crbToGather := []string{"system:openshift:scc:anyuid", "system:openshift:scc:privileged"}
+
+		datahubsResource := schema.GroupVersionResource{Group: "installers.datahub.sap.com", Version: "v1alpha1", Resource: "datahubs"}
+
+		datahubsList, err := i.dynamicClient.Resource(datahubsResource).List(i.ctx, metav1.ListOptions{})
+
+		if errors.IsNotFound(err) {
+			return nil, nil
+		}
+		if err != nil {
+			return nil, []error{err}
+		}
+		// If no DataHubs resource exists on the cluster, skip this gathering.
+		// This may already be handled by the IsNotFound check, but it's better to be sure.
+		if len(datahubsList.Items) == 0 {
+			return nil, nil
+		}
+
+		records := []record.Record{}
+
+		for _, name := range sccToGather {
+			scc, err := i.securityClient.SecurityContextConstraints().Get(i.ctx, name, metav1.GetOptions{})
+			if errors.IsNotFound(err) {
+				continue
+			}
+			if err != nil {
+				return nil, []error{err}
+			}
+			records = append(records, record.Record{
+				Name: fmt.Sprintf("config/securitycontextconstraint/%s", scc.Name),
+				// It is not possible to use the generic OpenShift Anonymizer type here
+				// because the SCC and CRB resources returned by their respective clients
+				// are currently missing some properties (kind, apiVersion).
+				Item: record.JSONMarshaller{Object: scc},
+			})
+		}
+
+		for _, name := range crbToGather {
+			crb, err := i.authClient.ClusterRoleBindings().Get(i.ctx, name, metav1.GetOptions{})
+			if errors.IsNotFound(err) {
+				continue
+			}
+			if err != nil {
+				return nil, []error{err}
+			}
+			records = append(records, record.Record{
+				Name: fmt.Sprintf("config/clusterrolebinding/%s", strings.ReplaceAll(crb.Name, ":", "_")),
+				// See the note above on why it is not possible to use the generic Anonymizer type.
+				Item: record.JSONMarshaller{Object: crb},
+			})
+		}
+
+		return records, nil
+	}
+}
+
 func collectInstallPlan(recs map[string]*collectedPlan, item interface{}) []error {
 	// Get common prefix
 	csv := "[NONE]"
@@ -1381,6 +1521,60 @@ type contextKey string
 
 const contextKeyAllNamespaces contextKey = "allnamespaces"
 
+// GatherServiceAccounts collects ServiceAccount stats
+// from kubernetes default and namespaces starting with openshift.
+//
+// The Kubernetes api https://github.com/kubernetes/client-go/blob/master/kubernetes/typed/core/v1/serviceaccount.go#L83
+// Response see https://docs.openshift.com/container-platform/4.3/rest_api/index.html#serviceaccount-v1-core
+//
+// Location of serviceaccounts in archive: config/serviceaccounts
+// See: docs/insights-archive-sample/config/serviceaccounts
+func GatherServiceAccounts(i *Gatherer) func() ([]record.Record, []error) {
+	return func() ([]record.Record, []error) {
+		config, err := i.coreClient.Namespaces().List(i.ctx, metav1.ListOptions{Limit: maxServiceAccountNamespacesLimit})
+		if errors.IsNotFound(err) {
+			return nil, nil
+		}
+		if err != nil {
+			return nil, []error{err}
+		}
+		totalServiceAccounts := 0
+		serviceAccounts := []corev1.ServiceAccount{}
+		records := []record.Record{}
+		namespaces := defaultNamespaces
+		namespaceCollected := sets.NewString()
+		// collect from all openshift* namespaces + kubernetes defaults
+		for _, item := range config.Items {
+			if strings.HasPrefix(item.Name, "openshift") {
+				namespaces = append(namespaces, item.Name)
+			}
+		}
+		for _, namespace := range namespaces {
+			// fetching service accounts from namespace
+			if namespaceCollected.Has(namespace) {
+				continue
+			}
+			svca, err := i.coreClient.ServiceAccounts(namespace).List(i.ctx, metav1.ListOptions{Limit: maxServiceAccountsLimit})
+			if err != nil {
+				klog.V(2).Infof("Unable to read ServiceAccounts in namespace %s error %s", namespace, err)
+				continue
+			}
+
+			totalServiceAccounts += len(svca.Items)
+			for _, j := range svca.Items {
+				if len(serviceAccounts) > maxServiceAccountsLimit {
+					break
+				}
+				serviceAccounts = append(serviceAccounts, j)
+			}
+			namespaceCollected.Insert(namespace)
+		}
+
+		records = append(records, record.Record{Name: fmt.Sprintf("config/serviceaccounts"), Item: ServiceAccountsMarshaller{serviceAccounts, totalServiceAccounts}})
+		return records, nil
+	}
+}
+
 // RawByte is skipping Marshalling from byte slice
 type RawByte []byte
 
@@ -1793,20 +1987,11 @@ func (a ConfigMapAnonymizer) GetExtension() string {
 	return ""
 }
 
-// HostSubnetAnonymizer implements HostSubnet serialization wiht anonymization
+// HostSubnetAnonymizer implements HostSubnet serialization
 type HostSubnetAnonymizer struct{ *networkv1.HostSubnet }
 
 // Marshal implements HostSubnet serialization
 func (a HostSubnetAnonymizer) Marshal(_ context.Context) ([]byte, error) {
-	a.HostSubnet.HostIP = anonymizeString(a.HostSubnet.HostIP)
-	a.HostSubnet.Subnet = anonymizeString(a.HostSubnet.Subnet)
-
-	for i, s := range a.HostSubnet.EgressIPs {
-		a.HostSubnet.EgressIPs[i] = networkv1.HostSubnetEgressIP(anonymizeString(string(s)))
-	}
-	for i, s := range a.HostSubnet.EgressCIDRs {
-		a.HostSubnet.EgressCIDRs[i] = networkv1.HostSubnetEgressCIDR(anonymizeString(string(s)))
-	}
 	return runtime.Encode(networkSerializer.LegacyCodec(networkv1.SchemeGroupVersion), a.HostSubnet)
 }
 
@@ -2073,3 +2258,91 @@ func (a InstallPlanAnonymizer) Marshal(_ context.Context) ([]byte, error) {
 func (a InstallPlanAnonymizer) GetExtension() string {
 	return "json"
 }
+
+// findPVByPVCName tries to find *corev1.PersistentVolume used in PersistentVolumeClaim with provided name
+func findPVByPVCName(ctx context.Context, coreClient corev1client.CoreV1Interface, name string) (*corev1.PersistentVolume, error) {
+	// unfortunately we can't do "coreClient.PersistentVolumeClaims("").Get(ctx, name, ... )"
+	pvcs, err := coreClient.PersistentVolumeClaims("").List(ctx, metav1.ListOptions{})
+	if err != nil {
+		return nil, err
+	}
+	var pvc *corev1.PersistentVolumeClaim
+	for _, p := range pvcs.Items {
+		if p.Name == name {
+			pvc = &p
+			break
+		}
+	}
+	if pvc == nil {
+		return nil, fmt.Errorf("can't find any %s persistentvolumeclaim", name)
+	}
+	pvName := pvc.Spec.VolumeName
+	pv, err := coreClient.PersistentVolumes().Get(ctx, pvName, metav1.GetOptions{})
+	if err != nil {
+		return nil, err
+	}
+	return pv, nil
+}
+
+// PersistentVolumeAnonymizer implements serialization with marshalling
+type PersistentVolumeAnonymizer struct {
+	*corev1.PersistentVolume
+}
+
+// Marshal implements serialization of corev1.PersistentVolume without anonymization
+func (p PersistentVolumeAnonymizer) Marshal(_ context.Context) ([]byte, error) {
+	return runtime.Encode(kubeSerializer, p.PersistentVolume)
+}
+
+// GetExtension returns extension for PersistentVolume objects
+func (p PersistentVolumeAnonymizer) GetExtension() string {
+	return "json"
+}
+
+// NetNamespaceAnonymizer implements NetNamespace serialization
+type NetNamespaceAnonymizer struct{ namespaces []*netNamespace }
+
+// Marshal implements NetNamespace serialization
+func (a NetNamespaceAnonymizer) Marshal(_ context.Context) ([]byte, error) {
+	return json.Marshal(a.namespaces)
+}
+
+// GetExtension returns extension for NetNamespace object
+func (a NetNamespaceAnonymizer) GetExtension() string {
+	return "json"
+}
+
+// ServiceAccountsMarshaller implements serialization of Service Accounts
+type ServiceAccountsMarshaller struct {
+	sa                   []corev1.ServiceAccount
+	totalServiceAccounts int
+}
+
+// Marshal implements serialization of ServiceAccount
+func (a ServiceAccountsMarshaller) Marshal(_ context.Context) ([]byte, error) {
+	// Creates map for marshal
+	sr := map[string]interface{}{}
+	st := map[string]interface{}{}
+	st["TOTAL_COUNT"] = a.totalServiceAccounts
+	sr["serviceAccounts"] = st
+	nss := map[string]interface{}{}
+	st["namespaces"] = nss
+	for _, sa := range a.sa {
+		var ns map[string]interface{}
+		var ok bool
+		if _, ok = nss[sa.Namespace]; !ok {
+			ns = map[string]interface{}{}
+			nss[sa.Namespace] = ns
+		} else {
+			ns = nss[sa.Namespace].(map[string]interface{})
+		}
+		ns["name"] = sa.Name
+		ns["secrets"] = len(sa.Secrets)
+	}
+	return json.Marshal(sr)
+}
+
+// GetExtension returns extension for anonymized openshift objects
+func (a ServiceAccountsMarshaller) GetExtension() string {
+	return "json"
+}
