@@ -9,10 +9,14 @@ import (
 	"reflect"
 	"testing"
 
+	authv1 "github.com/openshift/api/authorization/v1"
 	configv1 "github.com/openshift/api/config/v1"
 	imageregistryv1 "github.com/openshift/api/imageregistry/v1"
 	networkv1 "github.com/openshift/api/network/v1"
+	securityv1 "github.com/openshift/api/security/v1"
+	authfake "github.com/openshift/client-go/authorization/clientset/versioned/fake"
 	configfake "github.com/openshift/client-go/config/clientset/versioned/fake"
+	securityfake "github.com/openshift/client-go/security/clientset/versioned/fake"
 	corev1 "k8s.io/api/core/v1"
 	policyv1beta1 "k8s.io/api/policy/v1beta1"
 	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
@@ -564,11 +568,11 @@ func TestGatherHostSubnet(t *testing.T) {
 	if err != nil {
 		t.Fatalf("failed to decode object: %v", err)
 	}
-	if gatheredHostSubnet.HostIP != "xxxxxxxx" {
-		t.Fatalf("Host IP is not anonymized %s", gatheredHostSubnet.HostIP)
+	if gatheredHostSubnet.HostIP != testHostSubnet.HostIP {
+		t.Fatalf("unexpected Host IP value %s", gatheredHostSubnet.HostIP)
 	}
-	if gatheredHostSubnet.Subnet != "xxxxxxxxxxx" {
-		t.Fatalf("Host Subnet is not anonymized %s", gatheredHostSubnet.Subnet)
+	if gatheredHostSubnet.Subnet != testHostSubnet.Subnet {
+		t.Fatalf("unexpected Subnet value %s", gatheredHostSubnet.Subnet)
 	}
 	if len(gatheredHostSubnet.EgressIPs) != len(testHostSubnet.EgressIPs) {
 		t.Fatalf("unexpected number of egress IPs gathered %s", gatheredHostSubnet.EgressIPs)
@@ -578,17 +582,6 @@ func TestGatherHostSubnet(t *testing.T) {
 		t.Fatalf("unexpected number of egress CIDRs gathered %s", gatheredHostSubnet.EgressCIDRs)
 	}
 
-	for _, ip := range gatheredHostSubnet.EgressIPs {
-		if ip != "xxxxxxxx" {
-			t.Fatalf("Egress IP is not anonymized %s", ip)
-		}
-	}
-
-	for _, cidr := range gatheredHostSubnet.EgressCIDRs {
-		if cidr != "xxxxxxxxxxx" {
-			t.Fatalf("Egress CIDR is not anonymized %s", cidr)
-		}
-	}
 }
 
 func TestGatherMachineSet(t *testing.T) {
@@ -627,7 +620,6 @@ metadata:
 	}
 }
 
-
 func TestGatherClusterOperator(t *testing.T) {
 	testOperator := &configv1.ClusterOperator{
 		ObjectMeta: metav1.ObjectMeta{
@@ -655,6 +647,7 @@ func TestGatherClusterOperator(t *testing.T) {
 		t.Fatalf("unexpected clusteroperator name %s", gatheredCO.Name)
 	}
 }
+
 func TestGatherMachineConfigPool(t *testing.T) {
 	var machineconfigpoolYAML = `
 apiVersion: machineconfiguration.openshift.io/v1
@@ -827,6 +820,214 @@ metadata:
 	}
 }
 
+func TestGatherNetNamespaces(t *testing.T) {
+	ns1 := &networkv1.NetNamespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "test-namespaces-1",
+		},
+		EgressIPs: []networkv1.NetNamespaceEgressIP{"10.10.10.10"},
+		NetID:     12345,
+	}
+	ns2 := &networkv1.NetNamespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "test-namespaces-2",
+		},
+		EgressIPs: []networkv1.NetNamespaceEgressIP{"11.11.11.11"},
+		NetID:     67891,
+	}
+	ctx := context.Background()
+	cs := networkfake.NewSimpleClientset()
+
+	gatherer := &Gatherer{networkClient: cs.NetworkV1()}
+	createNetNamespaces(ctx, t, cs, ns1, ns2)
+	rec, errs := GatherNetNamespace(gatherer)()
+	if len(errs) > 0 {
+		t.Errorf("unexpected errors: %#v", errs)
+		return
+	}
+	if len(rec) != 1 {
+		t.Fatalf("unexpected number or records %d", len(rec))
+	}
+	it1 := rec[0].Item
+	it1Bytes, err := it1.Marshal(context.TODO())
+	if err != nil {
+		t.Fatalf("unable to marshal: %v", err)
+	}
+	var netNamespaces []netNamespace
+	err = json.Unmarshal(it1Bytes, &netNamespaces)
+	if err != nil {
+		t.Fatalf("failed to decode object: %v", err)
+	}
+	if len(netNamespaces) != 2 {
+		t.Fatalf("unexpected number of namespaces gathered %d", len(netNamespaces))
+	}
+	if !equalNetNamespaceS(*ns1, netNamespaces[0]) {
+		t.Fatalf("unexpected netnamespace %v ", netNamespaces[0])
+	}
+
+	if !equalNetNamespaceS(*ns2, netNamespaces[1]) {
+		t.Fatalf("unexpected netnamespace %v ", netNamespaces[1])
+	}
+}
+
+func TestGatherServiceAccounts(t *testing.T) {
+	tests := []struct {
+		name string
+		data []*corev1.ServiceAccount
+		exp  string
+	}{
+		{
+			name: "one account",
+			data: []*corev1.ServiceAccount{&corev1.ServiceAccount{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "local-storage-operator",
+					Namespace: "default",
+				},
+				Secrets: []corev1.ObjectReference{corev1.ObjectReference{}},
+			}},
+			exp: `{"serviceAccounts":{"TOTAL_COUNT":1,"namespaces":{"default":{"name":"local-storage-operator","secrets":1}}}}`,
+		},
+		{
+			name: "multiple accounts",
+			data: []*corev1.ServiceAccount{&corev1.ServiceAccount{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "deployer",
+					Namespace: "openshift",
+				},
+				Secrets: []corev1.ObjectReference{corev1.ObjectReference{}},
+			},
+				&corev1.ServiceAccount{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      "openshift-apiserver-sa",
+						Namespace: "openshift-apiserver",
+					},
+					Secrets: []corev1.ObjectReference{corev1.ObjectReference{}},
+				}},
+			exp: `{"serviceAccounts":{"TOTAL_COUNT":2,"namespaces":{"openshift":{"name":"deployer","secrets":1},"openshift-apiserver":{"name":"openshift-apiserver-sa","secrets":1}}}}`,
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			coreClient := kubefake.NewSimpleClientset()
+			for _, d := range test.data {
+				_, err := coreClient.CoreV1().Namespaces().Create(context.Background(), &corev1.Namespace{ObjectMeta: metav1.ObjectMeta{Name: d.Namespace}}, metav1.CreateOptions{})
+				if err != nil {
+					t.Fatalf("unable to create fake ns %s", err)
+				}
+				_, err = coreClient.CoreV1().ServiceAccounts(d.Namespace).
+					Create(context.Background(), d, metav1.CreateOptions{})
+				if err != nil {
+					t.Fatalf("unable to create fake service account %s", err)
+				}
+			}
+			gatherer := &Gatherer{coreClient: coreClient.CoreV1()}
+			sa, errs := GatherServiceAccounts(gatherer)()
+			if len(errs) > 0 {
+				t.Fatalf("unexpected errors: %#v", errs)
+				return
+			}
+			bts, err := sa[0].Item.Marshal(context.Background())
+			if err != nil {
+				t.Fatalf("error marshalling %s", err)
+			}
+			s := string(bts)
+			if test.exp != s {
+				t.Fatalf("serviceaccount test failed. expected: %s got: %s", test.exp, s)
+			}
+		})
+	}
+}
+
+func TestGatherSAPConfig(t *testing.T) {
+	// Initialize the fake dynamic client.
+	var datahubYAML = `apiVersion: installers.datahub.sap.com/v1alpha1
+kind: DataHub
+metadata:
+    name: example-datahub
+    namespace: example-namespace
+`
+
+	datahubsResource := schema.GroupVersionResource{Group: "installers.datahub.sap.com", Version: "v1alpha1", Resource: "datahubs"}
+	datahubsClient := dynamicfake.NewSimpleDynamicClient(runtime.NewScheme())
+
+	decUnstructured := yaml.NewDecodingSerializer(unstructured.UnstructuredJSONScheme)
+	testDatahub := &unstructured.Unstructured{}
+
+	_, _, err := decUnstructured.Decode([]byte(datahubYAML), nil, testDatahub)
+	if err != nil {
+		t.Fatal("unable to decode datahub YAML", err)
+	}
+
+	// Initialize the remaining K8s/OS fake clients.
+	coreClient := kubefake.NewSimpleClientset()
+	authClient := authfake.NewSimpleClientset()
+	securityClient := securityfake.NewSimpleClientset()
+
+	// Security Context Constraints.
+	securityClient.SecurityV1().SecurityContextConstraints().Create(
+		context.Background(),
+		&securityv1.SecurityContextConstraints{ObjectMeta: metav1.ObjectMeta{Name: "anyuid"}},
+		metav1.CreateOptions{},
+	)
+	securityClient.SecurityV1().SecurityContextConstraints().Create(
+		context.Background(),
+		&securityv1.SecurityContextConstraints{ObjectMeta: metav1.ObjectMeta{Name: "privileged"}},
+		metav1.CreateOptions{},
+	)
+	// This SCC should not be collected.
+	securityClient.SecurityV1().SecurityContextConstraints().Create(
+		context.Background(),
+		&securityv1.SecurityContextConstraints{ObjectMeta: metav1.ObjectMeta{Name: "ignored"}},
+		metav1.CreateOptions{},
+	)
+
+	// Cluster Role Bindings.
+	authClient.AuthorizationV1().ClusterRoleBindings().Create(
+		context.Background(),
+		&authv1.ClusterRoleBinding{ObjectMeta: metav1.ObjectMeta{Name: "system:openshift:scc:anyuid"}},
+		metav1.CreateOptions{},
+	)
+	authClient.AuthorizationV1().ClusterRoleBindings().Create(
+		context.Background(),
+		&authv1.ClusterRoleBinding{ObjectMeta: metav1.ObjectMeta{Name: "system:openshift:scc:privileged"}},
+		metav1.CreateOptions{},
+	)
+	// This CRB should not be collected.
+	authClient.AuthorizationV1().ClusterRoleBindings().Create(
+		context.Background(),
+		&authv1.ClusterRoleBinding{ObjectMeta: metav1.ObjectMeta{Name: "system:openshift:scc:ignored"}},
+		metav1.CreateOptions{},
+	)
+
+	gatherer := &Gatherer{
+		ctx:            context.Background(),
+		dynamicClient:  datahubsClient,
+		coreClient:     coreClient.CoreV1(),
+		securityClient: securityClient.SecurityV1(),
+		authClient:     authClient.AuthorizationV1(),
+	}
+
+	records, errs := GatherSAPConfig(gatherer)()
+	if len(errs) > 0 {
+		t.Fatalf("unexpected errors: %#v", errs)
+	}
+	if len(records) != 0 {
+		t.Fatalf("unexpected number or records in the first run: %d", len(records))
+	}
+
+	// Create the DataHubs resource and now the SCCs and CRBs should be gathered.
+	datahubsClient.Resource(datahubsResource).Namespace("example-namespace").Create(context.Background(), testDatahub, metav1.CreateOptions{})
+
+	records, errs = GatherSAPConfig(gatherer)()
+	if len(errs) > 0 {
+		t.Fatalf("unexpected errors: %#v", errs)
+	}
+	if len(records) != 4 {
+		t.Fatalf("unexpected number or records in the second run: %d", len(records))
+	}
+}
+
 func ExampleGatherMostRecentMetrics_Test() {
 	b, err := ExampleMostRecentMetrics()
 	if err != nil {
@@ -874,3 +1075,27 @@ func findMap(cml *corev1.ConfigMapList, name string) *corev1.ConfigMap {
 	}
 	return nil
 }
+
+func equalNetNamespaceS(ns1 networkv1.NetNamespace, ns2 netNamespace) bool {
+	if ns1.Name != ns2.Name {
+		return false
+	}
+	if ns1.NetID != ns2.NetID {
+		return false
+	}
+	if !reflect.DeepEqual(ns1.EgressIPs, ns2.EgressIPs) {
+		return false
+	}
+	return true
+}
+
+func createNetNamespaces(ctx context.Context, t *testing.T, n *networkfake.Clientset, namespaces ...*networkv1.NetNamespace) {
+	for _, ns := range namespaces {
+		_, err := n.NetworkV1().NetNamespaces().Create(ctx, ns, metav1.CreateOptions{})
+		if err != nil {
+			if err != nil {
+				t.Fatal("unable to create fake netnamespace", err)
+			}
+		}
+	}
+}
